"""
            ClickTracker product (clickthrough logging)
	
    Copyright (c) 1999 Coltronix Europe Corp. (http://www.coltronix.com)
    Written by Petru Paler (ppetru@coltronix.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

"""

import Acquisition
import AccessControl
import OFS
from Globals import HTMLFile, MessageDialog, Persistent
from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from Products.ZSQLMethods.SQL import SQL

class ClickTracker(
	Acquisition.Implicit,
	Persistent,
	AccessControl.Role.RoleManager,
	OFS.SimpleItem.Item
	):
	"""
	Clickthrough logging system
	"""

	name='ClickTracker'
	id='ClickTracker'
	icon='misc_/ClickTracker/Main'
	meta_type='ClickTracker'

	manage_options=(
		{'label':'Contents','action':'manage_main'},
		{'label':'Security','action':'manage_access'},
		{'label':'About', 'action':'manage_about'},
		)

	__ac_permissions__=(
		('View management screens',('manage_tabs','manage_main')),
		('Change permissions',('manage_access',)),
		('Change ClickTracker',('manage_edit',),('Manager',)),
		)

	def __init__(self, conn_id):
		# save the SQL connection id (the one that the DTML form has passed us)
		self.conn_id = conn_id

		# create the SQL method that we use for logging (source is at the bottom of this file)
		self.sqlClickLog = SQL('sqlClickLog', '', conn_id, 'local_url foreign_url', _sqlClickLog)
					# name, title, connection id, parameters, source

	def enumQueries(self):
		"""
		List the SQL queries used by this product
		"""
		list = []
		type = SQL.meta_type
		for ob in self.__dict__.values():
			if hasattr(ob, 'meta_type') and ob.meta_type==type:
				list.append(ob)
		return list

	def manage_edit(self, connection_id, REQUEST=None):
		"""Change properties"""

		if connection_id != self.conn_id:
			# connection changed - update SQL methods
			self.conn_id = connection_id
			for ob in self.enumQueries():
				title = ob.title
				args = ob.arguments_src
				src = ob.src
				ob.manage_edit(title, self.conn_id, args, src)
		if REQUEST:
			return MessageDialog(
				title='ClickTracker changed',
				message='ClickTracker properties have benn updated',
				action=REQUEST['URL1']+'/manage_properties',
				target='manage_main')

	def index_html(self, REQUEST, RESPONSE, t):
		"""
		The real work is done here.
		"""
		
		# redirect the user to the new URL before doing anything else
		# this way he won't have to wait for the database insert to complete
		RESPONSE.redirect(t)
		# log the clickthrough to the database
		# we don't care about the result (which is empty anyway)
		r = self.sqlClickLog(local_url=REQUEST['URL2'], foreign_url=t)


	# the "Contents" management tab
	manage_main = HTMLFile('main', globals())
	# the "About" tab
	manage_about = HTMLFile('About', globals()) 

# the add form
addClickTrackerForm=HTMLFile('addClickTracker',globals())

def addClickTracker(self, connection_id, REQUEST=None):
	"""Create a ClickTracker and install it in its parent Folder
	The argument 'self' will be bound to the parent Folder.
	"""

	# since the id is fixed we can't have two ClickTrackers in the same folder
	if hasattr(self.aq_base, 'r'):
 		return MessageDialog(
  	     		title  ='Item Exists',
   			message='This object already contains a ClickTracker',
    	  		action ='%s/manage_main' % REQUEST['URL1'])

	# create the object instance
	handle=ClickTracker(connection_id)
	# and stick it in the parent folder
	self._setObject('r',handle)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)

# the SQL source

_sqlClickLog = """<dtml-comment>
This SQL method logs a clickthrough.
</dtml-comment>
INSERT INTO clickthrough_log
(local_url, foreign_url, entry_date)
VALUES
(<dtml-sqlvar local_url type=string>, <dtml-sqlvar foreign_url type=string>, now())
"""
